--[[============================================================================
main.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value


--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Key Handler Example",
  invoke = function() handle_key_events() end
}

function handle_key_events()

  local vb = renoise.ViewBuilder()

  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local CONTENT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING

  local TEXT_ROW_WIDTH = 240
  
  local content_view =  vb:column {
    margin = DIALOG_MARGIN,
    spacing = CONTENT_SPACING,

    vb:text {
      width = TEXT_ROW_WIDTH,
      text = "Press some keyboard keys:"
    },
    
    vb:row {
      style = "group",
      
      vb:multiline_text {
        id = "key_text",
        width = TEXT_ROW_WIDTH,
        height = 78,
        paragraphs = {
          "key.name:", 
          "key.modifiers:", 
          "key.character:", 
          "key.note:",
          "key.repeat:"
        },
        font = "mono",
      }
    }
  }
    
  local function key_handler(dialog, key)
  
    -- update key_text to show what we got
    vb.views.key_text.paragraphs = {
      ("key.name: '%s'"):format(key.name), 
      ("key.modifiers: '%s'"):format(key.modifiers), 
      ("key.character: '%s'"):format(key.character or "nil"), 
      ("key.note: '%s'"):format(tostring(key.note) or "nil"),
      ("key.repeated: '%s'"):format(key.repeated and "true" or "false")
    }

    if (key.modifiers == "" and key.name == "esc") then
    
      -- close on escape.
      dialog:close()
    
    elseif (key.modifiers == "" and key.name == "up") then
    
      -- move up a line.
      local edit_pos = renoise.song().transport.edit_pos
      edit_pos.line = edit_pos.line - 1
      if (edit_pos.line < 1) then
        edit_pos.line = renoise.song().selected_pattern.number_of_lines
      end
      renoise.song().transport.edit_pos = edit_pos  
         
    elseif (key.modifiers == "" and key.name == "down") then
      
      -- move down a line.
      local edit_pos = renoise.song().transport.edit_pos
      edit_pos.line = edit_pos.line + 1
      if (edit_pos.line > renoise.song().selected_pattern.number_of_lines) then
        edit_pos.line = 1
      end
      renoise.song().transport.edit_pos = edit_pos  
      
    else
    
      -- nothing special to do.
      return key
      
    end    
    
  end
  
  -- show a dialog as usual, but this time also pass a keyboard handler ref
  renoise.app():show_custom_dialog(
    "Handling Keyboard Events", content_view, key_handler)

end
